# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: table_schema.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12table_schema.proto\"\xcd\x01\n\x10table_schema_pb2\x12\x11\n\tCLIENT_ID\x18\x01 \x02(\t\x12\x12\n\nPATIENT_ID\x18\x02 \x02(\t\x12\x0b\n\x03MRN\x18\x03 \x02(\t\x12\x11\n\tFIRSTNAME\x18\x04 \x01(\t\x12\x10\n\x08LASTNAME\x18\x05 \x01(\t\x12\x12\n\nMIDDLENAME\x18\x06 \x01(\t\x12\x0b\n\x03SEX\x18\x07 \x01(\t\x12\x0c\n\x04RACE\x18\x08 \x01(\t\x12\x0b\n\x03\x44OB\x18\t \x01(\x05\x12\x10\n\x08\x43HECKSUM\x18\n \x01(\t\x12\x12\n\nUPDATED_AT\x18\x0b \x02(\t')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'table_schema_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TABLE_SCHEMA_PB2._serialized_start=23
  _TABLE_SCHEMA_PB2._serialized_end=228
# @@protoc_insertion_point(module_scope)
